// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:developer';

import 'package:authentication_bloc_example/constants.dart';
import 'package:authentication_bloc_example/home/home_page.dart';
import 'package:authentication_bloc_example/login/login_page.dart';
import 'package:authentication_bloc_example/model.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:wyatt_authentication_bloc/wyatt_authentication_bloc.dart';
import 'package:wyatt_form_bloc/wyatt_form_bloc.dart';

class App extends StatelessWidget {
  const App({Key? key}) : super(key: key);

  static FormData getNormalFormData() {
    return const FormData([
      FormInput(formFieldName, Name.pure()),
      FormInput(formFieldPhone, Phone.pure()),
      FormInput(formFieldPro, Boolean.pure()),
      FormInput(
        formFieldConfirmedPassword,
        ConfirmedPassword.pure(),
        metadata: FormInputMetadata(export: false),
      ),
    ]);
  }

  static FormData getProFormData() {
    return const FormData([
      FormInput(formFieldName, Name.pure()),
      FormInput(formFieldPhone, Phone.pure()),
      FormInput(formFieldPro, Boolean.pure()),
      FormInput(formFieldSiren, Siren.pure()),
      FormInput(formFieldIban, Iban.pure()),
      FormInput(
        formFieldConfirmedPassword,
        ConfirmedPassword.pure(),
        metadata: FormInputMetadata(export: false),
      ),
    ]);
  }

  // On Authentication Success. (Authenticated or Anonymous)
  // User callback.
  Future<Map<String, dynamic>> onAuthSuccess(UserInterface user) async {
    if (user.isNotEmpty && !user.isAnonymous) {
      // Check if user is register in Firesore.
      DocumentSnapshot firestoreUser = await FirebaseFirestore.instance
          .collection(firestoreCollectionUsers)
          .doc(user.uid)
          .get();

      if (!firestoreUser.exists) {
        // Register user in Firestore when sign in with social account.
        final uid = user.uid;
        final u = {'uid': uid, 'email': user.email};
        await FirebaseFirestore.instance
            .collection(firestoreCollectionUsers)
            .doc(uid)
            .set(u);
        return {
          'user': UserFirestore(
              uid: uid,
              email: user.email ?? '',
              name: user.displayName ?? '',
              phone: user.phoneNumber ?? ''),
        };
      } else {
        return {
          'user': UserFirestore.fromMap(
              firestoreUser.data() as Map<String, dynamic>),
          ...firestoreUser.data() as Map<String, dynamic>? ?? {}
        };
      }
    } else {
      return {};
    }
  }

  // On Sign Up Success.
  Future<void> onSignUpSuccess(SignUpState state, String? uid) async {
    if (uid != null) {
      final data = state.data.toMap();
      final user = {'uid': uid, 'email': state.email.value, ...data};
      log('onSignUpSuccess: $user');
      await FirebaseFirestore.instance
          .collection(firestoreCollectionUsers)
          .doc(uid)
          .set(user);
    }
  }

  @override
  Widget build(BuildContext context) {
    AuthenticationRepositoryInterface authenticationRepository =
        AuthenticationRepositoryFirebase();

    AuthenticationCubit authenticationCubit = AuthenticationCubit(
      authenticationRepository: authenticationRepository,
      onAuthSuccess: onAuthSuccess,
    );

    SignUpCubit signUpCubit = SignUpCubit(
      authenticationRepository: authenticationRepository,
      authenticationCubit: authenticationCubit,
      entries: getNormalFormData(),
      onSignUpSuccess: onSignUpSuccess,
    );

    SignInCubit signInCubit = SignInCubit(
      authenticationRepository: authenticationRepository,
      authenticationCubit: authenticationCubit,
    );

    return MultiRepositoryProvider(
      providers: [
        RepositoryProvider<AuthenticationRepositoryInterface>(
          create: (context) => authenticationRepository,
        ),
      ],
      child: MultiBlocProvider(
        providers: [
          BlocProvider<AuthenticationCubit>(
            create: (context) => authenticationCubit..init(),
          ),
          BlocProvider<SignUpCubit>(
            create: (context) => signUpCubit,
          ),
          BlocProvider<SignInCubit>(
            create: (context) => signInCubit,
          ),
        ],
        child: const AppView(),
      ),
    );
  }
}

class AppView extends StatelessWidget {
  const AppView({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      theme: ThemeData(
        primarySwatch: Colors.blue,
        buttonTheme: const ButtonThemeData(
          buttonColor: Colors.blue,
          textTheme: ButtonTextTheme.primary,
        ),
      ),
      home: AuthenticationBuilder(
        unknown: (context) {
          return const Scaffold(
            body: Center(
              child: CircularProgressIndicator(),
            ),
          );
        },
        authenticated: (context, user, userData) => const HomePage(),
        unauthenticated: (context) => const LoginPage(),
      ),
    );
  }
}
